@@ -1,28 +1,35 @@
 from .AI365VIP          import AI365VIP
-from .AIChatFree        import AIChatFree
+from .Aichat            import Aichat
 from .AiChatOnline      import AiChatOnline
 from .AiChats           import AiChats
-from .Airforce          import Airforce
-from .AutonomousAI      import AutonomousAI
+from .Ails              import Ails 
 from .AIUncensored      import AIUncensored
+from .AllenAI           import AllenAI
 from .AmigoChat         import AmigoChat
 from .Aura              import Aura
+from .ChatGpt           import ChatGpt
 from .Chatgpt4o         import Chatgpt4o
-from .Chatgpt4Online    import Chatgpt4Online
+from .ChatGptEs         import ChatGptEs
 from .ChatgptFree       import ChatgptFree
 from .ChatGptt          import ChatGptt
-from .DarkAI            import DarkAI
+from .DDG               import DDG
+from .Equing            import Equing
 from .FlowGpt           import FlowGpt
 from .FreeNetfly        import FreeNetfly
+from .FreeRouter        import FreeRouter
+from .Glider            import Glider
 from .GPROChat          import GPROChat
 from .Koala             import Koala
+from .Liaobots          import Liaobots
+from .Lockchat          import Lockchat
 from .MagickPen         import MagickPen
-from .MyShell           import MyShell
+from .Phind             import Phind
+from .Pizzagpt          import Pizzagpt
 from .Poe               import Poe
 from .Prodia            import Prodia
 from .Raycast           import Raycast
-from .ReplicateHome     import ReplicateHome
-from .RobocodersAPI     import RobocodersAPI
 from .RubiksAI          import RubiksAI
 from .Theb              import Theb
+from .TypeGPT           import TypeGPT
 from .Upstage           import Upstage
+from .Vercel            import Vercel
